﻿using System;
using System.Windows.Forms;

// Implements the logic of the game
// The displaying of the game is the responsibility of a separate class.
//
//
//
// Ian Smith                     26 Nov 2020       Original Version
//--------------------------------------------------------------------------------------------------------------------------
namespace TestGame.Game
{
    class SolveGame
    {
        private Game.GameLogic clsGameLogic;

        public Boolean SolveGame5xx5(Game.GameLogic GameLogic)
        {
            Boolean boolRes = false;

            try
            {
                // Preserve the logic class
                clsGameLogic = GameLogic;

                // Turn off rows 2 to 4;
                if (ScanRows(1, 3) == false) return false;

                // Use a fixed map on the 5th row
                if (ConstantMapRow5() == false) return false;

                // Again Turn off rows 2 to 4;
                if (ScanRows(1,3) == false) return false;

                // Set function return variable as a success
                boolRes = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return boolRes;

        }

        public Boolean ScanRows(Int16 StartRow, Int16 EndRow)
        {   // Start and end row are zero based indexes for row to scan
            Boolean boolRes = false;

            try
            {
                // Loop for grids width and height start at the second row and going down to the fourth row.
                for (Int16 lcY = StartRow; lcY <= EndRow; lcY++)
                {
                    for (Int16 lcX = 0; lcX < clsGameLogic.GridSizeX; lcX++)
                    {
                        // If the tile in the row above is set then toggle it in this row
                        if (clsGameLogic.GetTileStatus(lcX, (Int16)(lcY - 1)) == true)
                        {
                            if (clsGameLogic.ToggleTile(lcX, lcY, true) == false) return false;
                        }
                    }
                }

                // Set function return variable as a success
                boolRes = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return boolRes;

        }

        public Boolean ConstantMapRow5()
        {
            Boolean boolRes = false;
            Int16 TileToClick = -1;
            Int16 TileToClick2nd = -1;
            try
            {
                // 1,2,3 Set
                if ((clsGameLogic.GetTileStatus(4, 0) == false)
                 && (clsGameLogic.GetTileStatus(4, 1) == false)
                 && (clsGameLogic.GetTileStatus(4, 2) == false)) TileToClick = 1;

                // 1,2,4,5 Set
                if ((clsGameLogic.GetTileStatus(4, 0) == false)
                 && (clsGameLogic.GetTileStatus(4, 1) == false)
                 && (clsGameLogic.GetTileStatus(4, 3) == false)
                 && (clsGameLogic.GetTileStatus(4, 4) == false)) TileToClick = 2;

                // 1,3,4 Set
                if ((clsGameLogic.GetTileStatus(4, 0) == false)
                 && (clsGameLogic.GetTileStatus(4, 2) == false)
                 && (clsGameLogic.GetTileStatus(4, 3) == false)) TileToClick = 4;

                // 1,5 Set 
                if ((clsGameLogic.GetTileStatus(4, 0) == false)
                 && (clsGameLogic.GetTileStatus(4, 4) == false)) { TileToClick = 0; TileToClick2nd = 1; }

                // 2,3,5 Set
                if ((clsGameLogic.GetTileStatus(4, 1) == false)
                 && (clsGameLogic.GetTileStatus(4, 2) == false)
                 && (clsGameLogic.GetTileStatus(4, 4) == false)) TileToClick = 0;

                // 2,4 Set (0,4)
                if ((clsGameLogic.GetTileStatus(4, 1) == false)
                 && (clsGameLogic.GetTileStatus(4, 3) == false)) { TileToClick = 0; TileToClick2nd = 3; }

                // 3,4,5 Set
                if ((clsGameLogic.GetTileStatus(4, 2) == false)
                 && (clsGameLogic.GetTileStatus(4, 3) == false)
                 && (clsGameLogic.GetTileStatus(4, 4) == false)) TileToClick = 3;

                // Do we need to set any buttons
                if (TileToClick != -1)
                {   // Yes so click on that tile
                    if (clsGameLogic.ToggleTile(TileToClick, 0, true) == false) return false;
                }
                if (TileToClick2nd != -1)
                {   // Yes so click on that tile
                    if (clsGameLogic.ToggleTile(TileToClick2nd, 0, true) == false) return false;
                }

                // Did we get a map entry
                if ((TileToClick2nd == -1) && (TileToClick == -1))
                {   // No
                    MessageBox.Show("No map entry in ConstantMapRow5()", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return false;
                }

                boolRes = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return boolRes;

        }
    }
}