﻿using System;
using System.Windows.Forms;

// The one and only user interface form
//
//
//
// Ian Smith                     26 Nov 2020       Original Version
//-----------------------------------------------------------------------------------------------------------------------
namespace TestGame
{
    public partial class formMain : Form
    {
        // Classes that control the display and the logic of the game
        Game.GameLogic clsGameLogic = null;

        public formMain()
        {
            InitializeComponent();
        }

        private void menuAboutVersion_Click(object sender, EventArgs e)
        {
            // Show a dialog box woith information about this program and its version number
            About.formAbout frmAbout;
            try
            {
                // Create an instance of the about form and show it as a modal dialog
                frmAbout = new About.formAbout();
                frmAbout.ShowDialog();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void menuFileExit_Click(object sender, EventArgs e)
        {
            // Close the application requested.

            try
            {
                // Get user confirmation
                if (MessageBox.Show("Please Cofirm - Close the game", "Close?", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {   // Close application
                    Application.Exit();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttStartNewGame_Click(object sender, EventArgs e)
        {   // Start a new game button pressed.
            try
            {
                // Start a new game
                if (StartANewGame() == false)
                {   // Failed
                    MessageBox.Show("Failed to start the game", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        private Boolean StartANewGame()
        {
            Boolean boolRes = false;
            try
            {
                // If there are no instances of the game logic class then create it with a 5x5 grid
                if (clsGameLogic == null) clsGameLogic = new Game.GameLogic(5, 5); else clsGameLogic.ResetGame();

                // setup the game area
                CreateGameDisplay(clsGameLogic.GridSizeX, clsGameLogic.GridSizeY, this.boxGameArea);

                // Uodate the game display
                if (RefeshGameDisplay() == false)
                {   // Display failed
                    MessageBox.Show("Failed to update game screen", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return false;
                }

                // Disable the new game button and enable the solve and test button
                buttStartNewGame.Enabled = false;
                buttAutoSolve.Enabled = true;
                buttTest.Enabled = true;

                // Set function return variable as a success
                boolRes = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return boolRes;
        }

        private Boolean RefeshGameDisplay()
        {   // Updates the display class with the values in the logic class
            Boolean boolRes = false;
            try
            {
                // Loop for all of the tiles
                for (Int16 lcY = 0; lcY < GridSizeY; lcY++)
                {
                    for (Int16 lcX = 0; lcX < GridSizeY; lcX++)
                    {   // Call the display class to update a single tile
                        SetTileColour(boxGameArea, lcX, lcY, clsGameLogic.GetTileStatus(lcX, lcY));
                    }
                }

                // Now check if all of the tiles are on
                if (clsGameLogic.IsGameOver == true)
                {   // Yes they are, so enable the new game button
                    buttStartNewGame.Enabled = true;

                    // And display a message
                    MessageBox.Show("Game Over", "Game Over", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }

                // Set function return variable as a success
                boolRes = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return boolRes;
        }

        private void buttTest_Click(object sender, EventArgs e)
        {   // Calls a test function to complete the game, as it is a test button we don't care about error handling

            try
            {
                clsGameLogic.TestFinishGame();
                RefeshGameDisplay();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        public void MouseTile_Click(object sender, EventArgs e)
        {   // Mouse 
            Int16 TileX = 0;
            Int16 TileY = 0;

            try
            {
                // Get the tile that was clicked on
                if (GetTileFromMouseClick((PictureBox)sender, ref TileX, ref TileY) == false)
                {
                    MessageBox.Show("There was an error getting the tile position", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                // Pass the tile position to the update logic
                clsGameLogic.ToggleTile(TileX, TileY, true);

                // Now update the game tiles display and check for end of game
                if (RefeshGameDisplay() == false)
                {
                    MessageBox.Show("There was an error updating the tiles", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttAutoSolve_Click(object sender, EventArgs e)
        {   // Automatically solve the grid
            Game.SolveGame clsSolveGame;
            try
            {
                // Creat an instance of the solution close and solve the grid
                clsSolveGame = new Game.SolveGame();
                if (clsSolveGame.SolveGame5xx5(clsGameLogic) == false)
                {   // Falied
                    MessageBox.Show("Failed to automatically solve the puzzle", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                // Now update the game tiles display and check for end of game which it should be
                if (RefeshGameDisplay() == false)
                {
                    MessageBox.Show("There was an error updating the tiles", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }

                // Worry about it not being end of game.
                if (clsGameLogic.IsGameOver == false)
                {
                    MessageBox.Show("There was a failure in automatically completing the game", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }
    }
}
